@@ -1,16 +1,17 @@
 """This module holds the utility functions and constants for the lazuli package.
 
-Copyright 2020 TEAM SPIRIT. All rights reserved.
+Copyright 2022 TEAM SPIRIT. All rights reserved.
 Use of this source code is governed by a AGPL-style license that can be found
 in the LICENSE file.
-Refer to database.py or the project wiki on GitHub for usage examples.
+Refer to `database.py` or the project wiki on GitHub for usage examples.
 """
+from typing import Any
 import mysql.connector as con
 
 # CONSTANTS -------------------------------------------------------------------
 # Dictionary that maps inventory tabs' names to
 # their corresponding index in the DB/source
-map_inv_types = {
+MAP_INV_TYPES = {
 	'equipped': -1,
 	'equip': 1,
 	'eqp': 1,
@@ -19,31 +20,34 @@ map_inv_types = {
 	'etc': 4,
 	'setup': 3,  # Default name for Lazuli purposes
 	'install': 3,  # name in WZ
-	'cash': 5
+	'cash': 5,
 }
 
 
 # UTILITY FUNCTIONS -----------------------------------------------------------
-def get_key(dictionary, val):
+def get_key(dictionary: dict, val: Any) -> Any:
 	"""Generic function to return the key for a given value
 
 	Iterates through the dictionary, comparing values to see if it matches
 	the desired value. If so, return the corresponding key. If no matches are
-	found by the end, return False.
+	found by the end, return `False`.
 	This function short-circuits (i.e. returns with the first match found).
 	Note: OrderedDict is no longer necessary for this as of Python 3.6,
 	as order is preserved automagically.
+	Note2: This function does not check whether the value type provided matches 
+	with the type of the value in the provided dictionary.
 
 	Args:
-		dictionary: Dictionary, representing the dictionary to be searched
-		val: Var, representing the desired/target value to search for
+
+		dictionary (`dict`): Represents the dictionary to be searched
+		val (`any`): Represents the desired/target value to search for
 
 	Returns:
-		Var, representing the corresponding key (if any)
-		Defaults to False, if none are found
+		A variable of `any` type, representing the corresponding key (if any).
+		Defaults to `False`, if none are found.
 
 	Raises:
-		Generic error for any failures
+		A generic error for any failures
 	"""
 	try:
 		for key, value in dictionary.items():
@@ -59,22 +63,23 @@ def get_key(dictionary, val):
 		return False
 
 
-def get_db_all_hits(config, query):
+def get_db_all_hits(config: dict[str, str], query: str) -> list:
 	"""Generic function for fetching all matching data from the DB
 
 	Generic top level function for fetching all matching data from DB,
 	using the provided DB config and query
 
 	Args:
-		config: Dictionary, representing database config attributes
-		query: String, representing SQL query
+
+		config (`dict`): Represents the database config attributes
+		query (`str`): Represents the SQL query to execute
 
 	Returns:
-		List of objects, representing the result of the provided SQL query,
+		A `list` of objects, representing the result of the provided SQL query,
 		using the provided DB connection attributes
 
 	Raises:
-		SQL Error 2003: Can't cannect to DB
+		SQL Error 2003: Can't connect to DB
 		WinError 10060: No response from DB
 		List index out of range: Wrong column name
 		Generic error as a final catch-all
@@ -102,34 +107,36 @@ def get_db_all_hits(config, query):
 		)
 
 
-def get_db_first_hit(config, query):
+def get_db_first_hit(config: dict[str, str], query: str) -> Any:
 	"""Generic function for fetching the first result from DB
 
-	This function grabs the first hit from get_db_all_hits;
-	errors handled in get_db_all_hits.
+	This function grabs the first hit from `get_db_all_hits`;
+	errors handled in `get_db_all_hits`.
 
 	Args:
-		config: Dictionary, representing database config attributes
-		query: String, representing SQL query
+
+		config (`dict`): Represents the database config attributes
+		query (`str`): Represents the SQL query to execute
 
 	Returns:
-		Var, representing first result
+		A variable of `any` type, representing first result
 	"""
 	return get_db_all_hits(config, query)[0]
 
 
-def get_stat_by_column(data, column):
+def get_stat_by_column(data: dict[str, Any], column: str) -> Any:
 	"""Fetches dictionary attribute by key (wrapper)
 
 	Args:
-		data: Dictionary, representing account or character attributes
-		column: String, representing column name in DB
+
+		data (`dict`): Represents the account or character attributes
+		column (`str`): Represents the column name in DB
 
 	Returns:
-		Int or String, representing user attribute queried
+		An `int` or `str`, representing user attribute queried
 
 	Raises:
-		Generic error on failure
+		A generic error on failure
 	"""
 	try:
 		return data[column]
@@ -139,15 +146,18 @@ def get_stat_by_column(data, column):
 		)
 
 
-def write_to_db(config, query):
+def write_to_db(config: dict[str, str], query: str) -> bool:
 	"""Performs write operations to DB using the provided DB config and query
 
+	### CAN ONLY BE SET WHEN SERVER IS OFF!
+
 	Args:
-		config: Dictionary, representing database config attributes
-		query: String, representing SQL query
+
+		config (`dict`): Represents the database config attributes
+		query (`str`): Represents the SQL query to execute
 
 	Returns:
-		A Boolean representing whether the operation was successful
+		A `bool` representing whether the operation was successful
 
 	Raises:
 		SQL Error 2003: Can't connect to DB
@@ -174,45 +184,30 @@ def write_to_db(config, query):
 		return False
 
 
-def get_inv_type_by_name(inv_string):
-	inv_type = map_inv_types.get(inv_string)
+def get_inv_type_by_name(inv_string: str) -> int:
+	"""`int`: Encode an inventory type using its common name"""
+	inv_type = MAP_INV_TYPES.get(inv_string)
 	return inv_type
 
 
-def get_inv_name_by_type(inv_type):  # Never used
-	inv_name = get_key(map_inv_types, inv_type)
+def get_inv_name_by_type(inv_type: int) -> str:  # Never used
+	"""`str`: Decode an inventory type using from its value"""
+	inv_name = get_key(MAP_INV_TYPES, inv_type)
 	return inv_name
 
 
-def has_item_in_inv_type(inv_type, item_id):
-	"""Checks whether the particular tab of the inventory has an item
-
-	Generic top level function used by Inventory::has_item_in_XXX() methods,
-	and the Inventory::is_equipping() method. Iterates through the dictionary
-	of items associated with the specified tab, and check if
-	the provided item ID can be found as a value.
+def extract_name(player_list: list[dict[str, Any]]) -> list[str]:
+	"""Extracts a `list` of players from SQL data, via the name column
 
 	Args:
-		inv_type: Inventory object, representing inventory tab to search
-		item_id: Int, representing the ID of the item to search for
+
+		player_list (`list[dict]`): Represents a list of all players
 
 	Returns:
-		Boolean, representing whether the specified item was found
-	"""
-	for bag_index in inv_type:
-		if inv_type[bag_index]['itemid'] == item_id:
-			return True
-	return False
+		A `list` of `str`, representing player names
 
-
-def extract_name(player_list):
-	"""Extracts a list of players from SQL data, via the name column
-
-	Args:
-		player_list: List of Dictionaries, representing list of all players
-
-	Returns:
-		List of Strings, representing player names
+	Raises:
+		RuntimeError: Improperly formatted or empty player list
 	"""
 	if not player_list[0]['name']:  # if empty or null; sanity check
 		raise RuntimeError("No players found!")
@@ -225,19 +220,26 @@ def extract_name(player_list):
 		return players
 
 
-def extract_name_and_value(player_list, column):
-	"""Extracts a list of players and their corresponding attribute
+def extract_name_and_value(
+	player_list: list[dict[str, Any]],
+	column: str,
+) -> list[tuple[str, Any]]:
+	"""Extracts a `list` of players and their corresponding attribute
 
-	Extracts a list of players and their corresponding attribute value from
+	Extracts a `list` of players and their corresponding attribute value from
 	SQL data, via the name column and another provided column
 
 	Args:
-		player_list: List of Dictionaries, representing list of all players
-		column: String, representing column name to extract
+
+		player_list (`list[dict]`): Represents a list of all players
+		column (`str`): Represents the column name to extract
 
 	Returns:
-		List of Tuples, representing player names and
+		A `list` of `tuple`, representing player names and
 		their corresponding values (e.g. level)
+
+	Raises:
+		RuntimeError: Improperly formatted or empty player list
 	"""
 	if not player_list[0]['name']:  # if empty or null; sanity check
 		raise RuntimeError("No such players found!")
